package com.zsc.edu.dify.modules.sse;

import lombok.AllArgsConstructor;
import org.springframework.http.MediaType;
import org.springframework.http.codec.ServerSentEvent;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Sinks;
import reactor.util.concurrent.Queues;

import java.time.Duration;

@AllArgsConstructor
@RestController
@RequestMapping("/api/rest/sse")
public class SseController {

    private final Sinks.Many<ServerSentEvent<String>> sink = Sinks.many().multicast().onBackpressureBuffer(Queues.SMALL_BUFFER_SIZE);

    @GetMapping(value = "/send", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    public Flux<ServerSentEvent<String>> sendSse() {
        return sink.asFlux()
                .doOnSubscribe(subscription -> {
                    // 模拟发送事件
                    sink.tryEmitNext(ServerSentEvent.builder("Hello, SSE!").build());
                    sink.tryEmitNext(ServerSentEvent.builder("This is a Server-Sent Event.").build());
                    sink.tryEmitNext(ServerSentEvent.builder("SSE is working.").build());
                })
                .delayElements(Duration.ofSeconds(1));
    }
}
